"use strict";
/**
 * Decode fields values from URI to cosmic link JSON format. This format differs
 * from Stellar transaction format: it is simpler, allow for federated address
 * and can be stringified/parsed without loss of information.
 *
 * For each of those functions, any error is recorded in the `conf` object
 * and HTML nodes are updated accordingly.
 *
 * @private
 * @exports decode
 */

var decode = exports;

var check = require("./check");

var normalize = require("./normalize");

var specs = require("./specs");

var status = require("./status");

decode.query = function (conf) {
  var query = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "?";
  if (query.substr(0, 1) !== "?") status.fail(conf, "Invalid query", "throw");
  query = query.substr(1);
  var operations = [];
  var tdesc = {}; // Backward compatibility with the old non-standard syntax, deprecated since
  // 2019-08-26. This adds `type=` at the beginning of the query when the first
  // parameter doesn't contains an `=` sign.

  if (query.match(/^\w+(&|$)/)) query = "type=".concat(query);
  var parser;
  var params = query.split("&");

  for (var index in params) {
    var entry = params[index];
    var field = entry.split("=", 1)[0];
    var value = entry.substring(field.length + 1);
    if (!field) continue;

    if (field === "type") {
      if (parser) {
        status.error(conf, "Query declares 'type' several times", "throw");
      } else if (value !== "transaction") {
        operations[0] = {
          type: value
        };
      }

      parser = value;
      continue;
    } else if (!parser) {
      status.error(conf, "Query doesn't declare 'type' in first position", "throw");
    }

    if (field === "operation") {
      operations.push({
        type: value
      });
      parser = "operation";
      continue;
    }

    var decoded = decode.field(conf, field, value); /// Multi-operation link.

    if (parser === "transaction") {
      tdesc[field] = decoded;
    } else if (parser === "operation") {
      operations[operations.length - 1][field] = decoded; /// One-operation link.
    } else {
      if (specs.isTransactionField(field)) {
        tdesc[field] = decoded;
      } else {
        operations[0][field] = decoded;
      }
    }
  }

  tdesc.operations = operations;
  normalize.tdesc(conf, tdesc);
  tdesc.operations.forEach(function (odesc) {
    return normalize.odesc(conf, odesc);
  });
  check.tdesc(conf, tdesc);
  return tdesc;
};
/******************************************************************************/

/**
 * Decode `value` accordingly to `field` type.
 *
 * @param {string} field
 * @param {string} value
 */


decode.field = function (conf, field, value) {
  var type = specs.fieldType[field];
  return type ? decode.type(conf, type, value) : value;
};
/**
 * Decode `value` using the decoding function for `type`.
 *
 * @param {string} type
 * @param {string} value
 */


decode.type = function (conf, type, value) {
  if (value) {
    value = decodeURIComponent(value);
    return process[type] ? process[type](conf, value) : value;
  } else {
    return "";
  }
};
/******************************************************************************/


var process = {};

process.asset = function (conf, asset) {
  var assetLower = asset.toLowerCase();

  if (assetLower === "xlm" || assetLower === "native") {
    return {
      code: "XLM"
    };
  } else {
    var temp = asset.split(":");
    var object = {
      code: temp[0],
      issuer: temp[1]
    };
    return object;
  }
};

process.assetsArray = function (conf, assetsList) {
  var strArray = assetsList.split(",");
  return strArray.map(function (entry) {
    return decode.asset(conf, entry);
  });
};

process.authorizeFlag = function (conf, flag) {
  // Backward compatibility (protocol =< 12)
  switch (flag) {
    case "false":
      return 0;

    case "true":
      return 1;
  }

  return Number(flag);
};

process["boolean"] = function (conf, string) {
  switch (string) {
    case "true":
      return true;

    case "false":
      return false;
  }
};

process.buffer = function (conf, string) {
  var matched = string.match(/(^[^:]*):/);
  var type = matched && matched[1];

  switch (type) {
    case "base64":
      return {
        type: type,
        value: string.substr(type.length + 1)
      };

    case "text":
      string = string.substr(type.length + 1);
    // Fallthrough

    default:
      return {
        type: "text",
        value: process.string(conf, string)
      };
  }
};

process.date = function (conf, string) {
  /// now + {minutes} syntactic sugar
  if (string.match(/^\+[0-9]+$/)) return string; /// Use UTC timezone by default.

  if (string.match(/T[^+]*[0-9]$/)) string += "Z";
  return new Date(string).toISOString();
};

process.memo = function (conf, string) {
  var matched = string.match(/(^[^:]*):/);
  var type = matched && matched[1];

  switch (type) {
    case "base64":
    case "id":
    case "hash":
    case "return":
      return {
        type: type,
        value: string.substr(type.length + 1)
      };

    case "text":
      string = string.substr(type.length + 1);
    // Fallthrough

    default:
      return {
        type: "text",
        value: process.string(conf, string)
      };
  }
};

process.price = function (conf, price) {
  var numerator = price.replace(/:.*/, "");
  var denominator = price.replace(/^[^:]*:/, "");
  if (numerator === denominator) return price;else return {
    n: +numerator,
    d: +denominator
  };
};

process.signer = function (conf, signer) {
  var temp = signer.split(":");
  var object = {
    weight: temp[0],
    type: temp[1],
    value: temp[2]
  };
  return object;
};

process.string = function (conf, string) {
  return string.replace(/\+/g, " ");
};

process.network = process.string;
/******************************************************************************/

/**
 * Provide dummy aliases for every other type for convenience & backward
 * compatibility.
 */

specs.types.forEach(function (type) {
  exports[type] = function (conf, value) {
    return decode.type(conf, type, value);
  };
});